@@ -1,3 +1,34 @@
+#
+# Copyright (c) 2025 Rawley Fowler
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
+#
+# 2. Redistributions in binary form must reproduce the copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
+#
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+#
+
 package Plack::Handler::H2;
 
 use strict;
@@ -6,81 +37,118 @@ use File::Temp;
 use Plack::Handler::H2::Writer;
 
 require XSLoader;
-our $VERSION = '0.0.2';
-XSLoader::load(__PACKAGE__, $VERSION);
+our $VERSION = '0.0.3';
+XSLoader::load( __PACKAGE__, $VERSION );
 
 sub new {
-    my ($class, %args) = @_;
+    my ( $class, %args ) = @_;
     return bless \%args, $class;
 }
 
 sub run {
-    my ($self, $app) = @_;
+    my ( $self, $app ) = @_;
 
-    die "Unsupported OS for Plack::Handler::H2" if $^O !~ /linux|darwin|freebsd|openbsd/i;
+    die "Unsupported OS for Plack::Handler::H2"
+      if $^O !~ /linux|darwin|freebsd|openbsd/i;
 
-    if (!defined $self->{ssl_cert_file} || !defined $self->{ssl_key_file}) {
+    if ( !defined $self->{ssl_cert_file} || !defined $self->{ssl_key_file} ) {
         my $cert_dir = File::Temp->newdir( CLEANUP => 1 );
-        warn("SSL certificate or key file not provided. Generating self-signed certificate.\n");
-        ($self->{ssl_cert_file}, $self->{ssl_key_file}) = $self->_generate_self_signed_cert($cert_dir);
-        warn("Generated self-signed certificate at $self->{ssl_cert_file} and key at $self->{ssl_key_file}\n");
-        warn("!!! WARNING !!! Self-signed certificates may not be trusted by clients.\n");
+        warn(
+"SSL certificate or key file not provided. Generating self-signed certificate.\n"
+        );
+        ( $self->{ssl_cert_file}, $self->{ssl_key_file} ) =
+          $self->_generate_self_signed_cert($cert_dir);
+        warn(
+"Generated self-signed certificate at $self->{ssl_cert_file} and key at $self->{ssl_key_file}\n"
+        );
+        warn(
+"!!! WARNING !!! Self-signed certificates may not be trusted by clients.\n"
+        );
     }
 
-    if (!$self->{port} && $self->{port} ne '0') {
+    if ( !$self->{port} && $self->{port} ne '0' ) {
         $self->{port} = 5000;
     }
 
-    my $res = ph2_run_wrapper($self, $app, {
-        ssl_cert => $self->{ssl_cert_file},
-        ssl_key  => $self->{ssl_key_file},
-        address  => $self->{host} // '0.0.0.0',
-        port     => $self->{port},
-        timeout  => $self->{timeout} // 120,
-        read_timeout => $self->{read_timeout} // 60,
-        write_timeout => $self->{write_timeout} // 60,
-        request_timeout => $self->{request_timeout} // 30,
-        max_request_body_size => $self->{max_request_body_size} // 10 * 1024 * 1024 # (10 MB)
-    });
+    my $res = ph2_run_wrapper(
+        $self, $app,
+        {
+            ssl_cert              => $self->{ssl_cert_file},
+            ssl_key               => $self->{ssl_key_file},
+            address               => $self->{host} // '0.0.0.0',
+            port                  => $self->{port},
+            timeout               => $self->{timeout}         // 120,
+            read_timeout          => $self->{read_timeout}    // 60,
+            write_timeout         => $self->{write_timeout}   // 60,
+            request_timeout       => $self->{request_timeout} // 30,
+            max_request_body_size => $self->{max_request_body_size}
+              // 10 * 1024 * 1024    # (10 MB)
+        }
+    );
     return $res;
 }
 
 sub _generate_self_signed_cert {
-    my ($self, $cert_dir) = @_;
+    my ( $self, $cert_dir ) = @_;
 
     my $cert_file = File::Temp->new( DIR => $cert_dir, SUFFIX => '.crt' );
-    my $key_file = File::Temp->new( DIR => $cert_dir, SUFFIX => '.key' );
+    my $key_file  = File::Temp->new( DIR => $cert_dir, SUFFIX => '.key' );
+
+    warn(
+"Attempting to generate self-signed certificate using mkcert if available...\n"
+    );
+    my $mkcert_check = `which mkcert`;
+    chomp($mkcert_check);
+    if ($mkcert_check) {
+        warn("mkcert found. Generating certificate with mkcert...\n");
+        system("mkcert -cert-file $cert_file -key-file $key_file localhost");
+        return ( $cert_file, $key_file );
+    }
+
+    warn(
+"mkcert not found. Falling back to OpenSSL for self-signed certificate generation...\n"
+    );
 
     my $openssl_check = `which openssl`;
     chomp($openssl_check);
     unless ($openssl_check) {
-        die "OpenSSL is not installed or not found in PATH. Cannot generate self-signed certificate.";
+        die
+"OpenSSL is not installed or not found in PATH. Cannot generate self-signed certificate.";
     }
 
-    my $cmd = "openssl req -x509 -newkey rsa:2048 -keyout $key_file -out $cert_file -days 365 -nodes -subj '/CN=localhost' 2>/dev/null";
+    my $cmd =
+"openssl req -x509 -newkey rsa:2048 -keyout $key_file -out $cert_file -days 365 -nodes -subj '/CN=localhost' 2>/dev/null";
     system($cmd);
 
-    return ($cert_file, $key_file);
+    return ( $cert_file, $key_file );
 }
 
 sub _responder {
-    my ($env, $session) = @_;
+    my ( $env, $session ) = @_;
     my $responder = sub {
         my $response = shift;
-        if (ref($response) ne 'ARRAY' || (@$response < 2 || @$response > 3)) {
+        if ( ref($response) ne 'ARRAY' || ( @$response < 2 || @$response > 3 ) )
+        {
             warn "Invalid PSGI response in responder";
-            return [500, ['Content-Type' => 'text/plain'], ['Internal Server Error: invalid response from application']];
+            return [
+                500,
+                [ 'Content-Type' => 'text/plain' ],
+                ['Internal Server Error: invalid response from application']
+            ];
         }
 
-        if (scalar @$response == 2) {
-            ph2_stream_write_headers_wrapper($env, $session, $response);
-            return Plack::Handler::H2::Writer->new({
-                response => $response,
-                writer => sub {
-                    my ($end_stream, $data) = @_;
-                    ph2_stream_write_data_wrapper($env, $session, $end_stream, $data);    
+        if ( scalar @$response == 2 ) {
+            ph2_stream_write_headers_wrapper( $env, $session, $response );
+            return Plack::Handler::H2::Writer->new(
+                {
+                    response => $response,
+                    writer   => sub {
+                        my ( $end_stream, $data ) = @_;
+                        ph2_stream_write_data_wrapper( $env, $session,
+                            $end_stream, $data );
+                    }
                 }
-            });
+            );
         }
 
         return $response;
